@@ -1,21 +1,14 @@
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#    http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-# implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the BSD License. See the LICENSE file in the root of this repository
+# for complete details.
 
 from __future__ import absolute_import, division, print_function
 
 import pytest
 
-from cryptography.hazmat.bindings.openssl.binding import Binding
+from cryptography.hazmat.bindings.openssl.binding import (
+    Binding, _get_libraries, _get_windows_libraries
+)
 
 
 class TestOpenSSL(object):
@@ -25,9 +18,6 @@ class TestOpenSSL(object):
         assert binding.lib
         assert binding.ffi
 
-    def test_is_available(self):
-        assert Binding.is_available() is True
-
     def test_crypto_lock_init(self):
         b = Binding()
         b.init_static_locks()
@@ -110,9 +100,11 @@ class TestOpenSSL(object):
         assert b.lib.SSL_OP_ALL > 0
         ctx = b.lib.SSL_CTX_new(b.lib.TLSv1_method())
         ctx = b.ffi.gc(ctx, b.lib.SSL_CTX_free)
+        current_options = b.lib.SSL_CTX_get_options(ctx)
         resp = b.lib.SSL_CTX_set_options(ctx, b.lib.SSL_OP_ALL)
-        assert resp == b.lib.SSL_OP_ALL
-        assert b.lib.SSL_OP_ALL == b.lib.SSL_CTX_get_options(ctx)
+        expected_options = current_options | b.lib.SSL_OP_ALL
+        assert resp == expected_options
+        assert b.lib.SSL_CTX_get_options(ctx) == expected_options
 
     def test_ssl_options(self):
         # Test that we're properly handling 32-bit unsigned on all platforms.
@@ -122,9 +114,11 @@ class TestOpenSSL(object):
         ctx = b.ffi.gc(ctx, b.lib.SSL_CTX_free)
         ssl = b.lib.SSL_new(ctx)
         ssl = b.ffi.gc(ssl, b.lib.SSL_free)
+        current_options = b.lib.SSL_get_options(ssl)
         resp = b.lib.SSL_set_options(ssl, b.lib.SSL_OP_ALL)
-        assert resp == b.lib.SSL_OP_ALL
-        assert b.lib.SSL_OP_ALL == b.lib.SSL_get_options(ssl)
+        expected_options = current_options | b.lib.SSL_OP_ALL
+        assert resp == expected_options
+        assert b.lib.SSL_get_options(ssl) == expected_options
 
     def test_ssl_mode(self):
         # Test that we're properly handling 32-bit unsigned on all platforms.
@@ -134,6 +128,22 @@ class TestOpenSSL(object):
         ctx = b.ffi.gc(ctx, b.lib.SSL_CTX_free)
         ssl = b.lib.SSL_new(ctx)
         ssl = b.ffi.gc(ssl, b.lib.SSL_free)
+        current_options = b.lib.SSL_get_mode(ssl)
         resp = b.lib.SSL_set_mode(ssl, b.lib.SSL_OP_ALL)
-        assert resp == b.lib.SSL_OP_ALL
-        assert b.lib.SSL_OP_ALL == b.lib.SSL_get_mode(ssl)
+        expected_options = current_options | b.lib.SSL_OP_ALL
+        assert resp == expected_options
+        assert b.lib.SSL_get_mode(ssl) == expected_options
+
+    def test_libraries(self):
+        assert _get_libraries("darwin") == ["ssl", "crypto"]
+        assert "ssleay32mt" in _get_libraries("win32")
+
+    def test_windows_static_dynamic_libraries(self):
+        assert "ssleay32mt" in _get_windows_libraries("static")
+
+        assert "ssleay32mt" in _get_windows_libraries("")
+
+        assert "ssleay32" in _get_windows_libraries("dynamic")
+
+        with pytest.raises(ValueError):
+            _get_windows_libraries("notvalid")
