@@ -1,15 +1,6 @@
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#    http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
-# implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the BSD License. See the LICENSE file in the root of this repository
+# for complete details.
 
 from __future__ import absolute_import, division, print_function
 
@@ -18,9 +9,6 @@ import collections
 import re
 from contextlib import contextmanager
 
-from pyasn1.codec.der import encoder
-from pyasn1.type import namedtype, univ
-
 import pytest
 
 import six
@@ -40,8 +28,6 @@ def select_backends(names, backend_list):
     if names is None:
         return backend_list
     split_names = [x.strip() for x in names.split(',')]
-    # this must be duplicated and then removed to preserve the metadata
-    # pytest associates. Appending backends to a new list doesn't seem to work
     selected_backends = []
     for backend in backend_list:
         if backend.name in split_names:
@@ -55,12 +41,13 @@ def select_backends(names, backend_list):
         )
 
 
-def check_for_iface(name, iface, item):
-    if name in item.keywords and "backend" in item.funcargs:
-        if not isinstance(item.funcargs["backend"], iface):
-            pytest.skip("{0} backend does not support {1}".format(
-                item.funcargs["backend"], name
-            ))
+def skip_if_empty(backend_list, required_interfaces):
+    if not backend_list:
+        pytest.skip(
+            "No backends provided supply the interface: {0}".format(
+                ", ".join(iface.__name__ for iface in required_interfaces)
+            )
+        )
 
 
 def check_backend_support(item):
@@ -83,22 +70,8 @@ def raises_unsupported_algorithm(reason):
     assert exc_info.value._reason is reason
 
 
-class _DSSSigValue(univ.Sequence):
-    componentType = namedtype.NamedTypes(
-        namedtype.NamedType('r', univ.Integer()),
-        namedtype.NamedType('s', univ.Integer())
-    )
-
-
-def der_encode_dsa_signature(r, s):
-    sig = _DSSSigValue()
-    sig.setComponentByName('r', r)
-    sig.setComponentByName('s', s)
-    return encoder.encode(sig)
-
-
-def load_vectors_from_file(filename, loader):
-    with cryptography_vectors.open_vector_file(filename) as vector_file:
+def load_vectors_from_file(filename, loader, mode="r"):
+    with cryptography_vectors.open_vector_file(filename, mode) as vector_file:
         return loader(vector_file)
 
 
@@ -624,3 +597,62 @@ def load_fips_ecdsa_signing_vectors(vector_data):
     if data is not None:
         vectors.append(data)
     return vectors
+
+
+def load_kasvs_dh_vectors(vector_data):
+    """
+    Loads data out of the KASVS key exchange vector data
+    """
+
+    result_rx = re.compile(r"([FP]) \(([0-9]+) -")
+
+    vectors = []
+    data = {
+        "fail_z": False,
+        "fail_agree": False
+    }
+
+    for line in vector_data:
+        line = line.strip()
+
+        if not line or line.startswith("#"):
+            continue
+
+        if line.startswith("P = "):
+            data["p"] = int(line.split("=")[1], 16)
+        elif line.startswith("Q = "):
+            data["q"] = int(line.split("=")[1], 16)
+        elif line.startswith("G = "):
+            data["g"] = int(line.split("=")[1], 16)
+        elif line.startswith("Z = "):
+            z_hex = line.split("=")[1].strip().encode("ascii")
+            data["z"] = binascii.unhexlify(z_hex)
+        elif line.startswith("XstatCAVS = "):
+            data["x1"] = int(line.split("=")[1], 16)
+        elif line.startswith("YstatCAVS = "):
+            data["y1"] = int(line.split("=")[1], 16)
+        elif line.startswith("XstatIUT = "):
+            data["x2"] = int(line.split("=")[1], 16)
+        elif line.startswith("YstatIUT = "):
+            data["y2"] = int(line.split("=")[1], 16)
+        elif line.startswith("Result = "):
+            result_str = line.split("=")[1].strip()
+            match = result_rx.match(result_str)
+
+            if match.group(1) == "F":
+                if int(match.group(2)) in (5, 10):
+                    data["fail_z"] = True
+                else:
+                    data["fail_agree"] = True
+
+            vectors.append(data)
+
+            data = {
+                "p": data["p"],
+                "q": data["q"],
+                "g": data["g"],
+                "fail_z": False,
+                "fail_agree": False
+            }
+
+    return vectors
